﻿using IrController.Model;
using IrController.Repository;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;

namespace IrController.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]/[action]")]
    public class ArduinoController : ControllerBase, IArduinoRepository
    {
        private readonly IArduinoRepository _arduinoRepository;
        public ArduinoController(IArduinoRepository arduinoRepository)
        {
            _arduinoRepository = arduinoRepository;
        }

        /// <summary>
        /// Return list of all Arduinos
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public List<ArduinoModel> GetArduinos()
        {
            return _arduinoRepository.GetArduinos();
        }

        /// <summary>
        /// Return single Arduino
        /// </summary>
        /// <param name="id">Id of Arduino</param>
        /// <returns></returns>
        [HttpGet]
        public ArduinoModel GetArduino(int id)
        {
            return _arduinoRepository.GetArduino(id);
        }

        /// <summary>
        /// Save New/Edited Arduino
        /// </summary>
        /// <param name="arduino">Arduino</param>
        /// <returns></returns>
        [HttpPost]
        public bool ManageArduino(ArduinoModel arduino)
        {
            return _arduinoRepository.ManageArduino(arduino);
        }

        /// <summary>
        /// Remove Arduino
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public string RemoveArduino(int id)
        {
            return _arduinoRepository.RemoveArduino(id);
        }

        /// <summary>
        /// Simple list of Arduinos for main page of controllers
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public List<ArduinoSimple> GetSimpleArduinos()
        {
            return _arduinoRepository.GetSimpleArduinos();
        }
    }
}
