﻿using IrController.Model;
using IrController.Repository;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;

namespace IrController.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]/[action]")]
    public class CodeController : ControllerBase, ICodeRepository 
    {
        private readonly ICodeRepository _codeRepository;

        public CodeController(ICodeRepository codeRepository)
        {
            _codeRepository = codeRepository;
        }

        /// <summary>
        /// All codes for main page of codes
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public List<CodeModel> GetAllCodes()
        {
            return _codeRepository.GetAllCodes();
        }

        /// <summary>
        /// Geting all codes, which is not assign to any device
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public List<CodeModel> GetAllMissingCodes()
        {
            return _codeRepository.GetAllMissingCodes();
        }

        /// <summary>
        /// Get of all codes for one device
        /// </summary>
        /// <param name="deviceId"> Device Id</param>
        /// <returns></returns>
        [HttpGet]
        public List<CodeModel> GetCoddesByDevice(int deviceId)
        {
            return _codeRepository.GetCoddesByDevice(deviceId);
        }

        /// <summary>
        /// Get single code by Id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public CodeModel GetCode(int id)
        {
            return _codeRepository.GetCode(id);
        }

        /// <summary>
        /// Get of all function for one device
        /// </summary>
        /// <param name="deviceId"></param>
        /// <returns></returns>
        [HttpGet]
        public List<DeviceFunctionModel> GetDeviceFunctions(int deviceId)
        {
            return _codeRepository.GetDeviceFunctions(deviceId);
        }

        /// <summary>
        /// Get single missing code by Id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public CodeModel GetMissingCode(int id)
        {
            return _codeRepository.GetMissingCode(id);
        }

        /// <summary>
        /// Save of new code send from Arduino sender
        /// </summary>
        /// <param name="com"> Command</param>
        /// <param name="add"> Address</param>
        /// <param name="code"> Protocol </param>
        /// <returns></returns>
        [HttpGet]
        public bool GetNewCode(string com, string add, string code)
        {
            return _codeRepository.GetNewCode(com, add, code);
        }

        /// <summary>
        /// Returning status of device for HAP
        /// </summary>
        /// <param name="codeId"></param>
        /// <returns></returns>
        [HttpGet]
        public string GetStatus(int codeId)
        {
            return _codeRepository.GetStatus(codeId);
        }

        /// <summary>
        /// Save Added/Edited code
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        [HttpPost]
        public bool ManageCode(CodeModel code)
        {
            return _codeRepository.ManageCode(code);
        }

        /// <summary>
        /// Save Added/Edited missing code
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        [HttpPost]
        public bool ManageMissingCode(CodeModel code)
        {
            return _codeRepository.ManageMissingCode(code);
        }

        /// <summary>
        /// Remove code by Id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public string RemoveCode(int id)
        {
            return _codeRepository.RemoveCode(id);
        }

        /// <summary>
        /// Remove missing code by Id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public string RemoveMissingCode(int id)
        {
            return _codeRepository.RemoveMissingCode(id);
        }

        /// <summary>
        /// Set brightness to device 
        /// </summary>
        /// <param name="value"> brightness in %</param>
        /// <param name="codeId"> code Id</param>
        /// <returns></returns>
        [HttpGet]
        public bool SetBrightness(int value, int codeId)
        {
            return _codeRepository.SetBrightness(value, codeId);
        }

        /// <summary>
        /// Set color to device 
        /// </summary>
        /// <param name="value"> RGB HEX color</param>
        /// <param name="codeId"> code Id</param>
        /// <returns></returns>
        [HttpGet]
        public bool SetColor(string value, int codeId)
        {
            return _codeRepository.SetColor(value, codeId);
        }

        /// <summary>
        /// Turn on or off device by code Id
        /// </summary>
        /// <param name="value">1 or 0</param>
        /// <param name="codeId"></param>
        /// <returns></returns>
        [HttpGet]
        public bool SetSwitch(int value, int codeId)
        {
            return _codeRepository.SetSwitch(value, codeId);
        }

        /// <summary>
        /// Save of new code send from Arduino sender
        /// </summary>
        /// <param name="com"> Command</param>
        /// <param name="add"> Address</param>
        /// <param name="irp"> Protocol </param>
        /// <returns></returns>
        [HttpGet]
        public bool WriteCommand(string com, string add, string irp)
        {
            return _codeRepository.WriteCommand(com, add, irp);
        }
    }
}
