﻿using IrController.Model;
using IrController.Repository;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;

namespace IrController.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]/[action]")]
    public class DeviceController : ControllerBase, IDeviceRepository
    {

        private readonly IDeviceRepository _deviceRepository;
        private readonly IHostingEnvironment _hostingEnvironment;
        public DeviceController(IDeviceRepository deviceRepository, IHostingEnvironment hostingEnvironment)
        {
            _deviceRepository = deviceRepository;
            _hostingEnvironment = hostingEnvironment;
        }

        /// <summary>
        /// Generate code for Arduino Receiver
        /// </summary>
        /// <param name="apiUrl">API Url</param>
        /// <param name="id">Arduino Id</param>
        /// <returns></returns>
        [HttpGet]
        public JsonReponse GetArduinoReceiver(string apiUrl, int id)
        {
            return _deviceRepository.GetArduinoReceiver(apiUrl, id);
        }

        /// <summary>
        /// Generate code for Arduino sender
        /// </summary>
        /// <param name="apiUrl"> API Url</param>
        /// <param name="id"> Arduino Id</param>
        /// <returns></returns>
        [HttpGet]
        public JsonReponse GetArduinoSender(string apiUrl, int id)
        {
            return _deviceRepository.GetArduinoSender(apiUrl, id);
        }

        /// <summary>
        /// Get single device by Id
        /// </summary>
        /// <param name="id"> Device Id</param>
        /// <returns></returns>
        [HttpGet]
        public DeviceModel GetDevice(int id)
        {
            return _deviceRepository.GetDevice(id);
        }

        /// <summary>
        /// Get all devices
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public List<DeviceModel> GetDevices()
        {
            return _deviceRepository.GetDevices();
        }

        /// <summary>
        /// Get all templates for devices
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public List<DeviceTemplateModel> GetDeviceTemplates()
        {
            return _deviceRepository.GetDeviceTemplates();
        }

        /// <summary>
        /// Generate JSON for HAP
        /// </summary>
        /// <param name="apiUrl"> API Url</param>
        /// <param name="id"> Device Ip</param>
        /// <returns></returns>
        [HttpGet]
        public JsonReponse GetHttpRgbPushAccessory(string apiUrl, int id)
        {
            return _deviceRepository.GetHttpRgbPushAccessory(apiUrl, id);
        }

        /// <summary>
        /// Save Add/Edit device
        /// </summary>
        /// <param name="device"></param>
        /// <returns></returns>
        [HttpPost]
        public bool ManageDevice(DeviceModel device)
        {
            return _deviceRepository.ManageDevice(device);
        }

        /// <summary>
        /// Remove device by Id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet]
        public string RemoveDevice(int id)
        {
            return _deviceRepository.RemoveDevice(id);
        }

    }
}
