﻿using IrController.Api.LocalStorage;
using IrController.Model.User;
using IrController.Repository;
using Microsoft.AspNetCore.Mvc;

namespace IrController.Api.Controllers
{
    [ApiController]
    [Route("api/[controller]/[action]")]
    public class UserController : ControllerBase, IUserRepository
    {
        private readonly IUserRepository _userRepository;
        private readonly MyCache _cache;
        public UserController(IUserRepository userRepository, MyCache cache)
        {
            _userRepository = userRepository;
            _cache = cache;
        }

        /// <summary>
        /// Save changed password
        /// </summary>
        /// <param name="changePassword"></param>
        /// <returns></returns>
        [HttpPost]
        public LogedUserModel ChangePassword(NewPasswordModel changePassword)
        {
            return _userRepository.ChangePassword(changePassword);
        }

        /// <summary>
        /// Login 
        /// </summary>
        /// <param name="loginModel"></param>
        /// <returns></returns>
        [HttpPost]
        public LogedUserModel Login(LoginModel loginModel)
        {
            LogedUserModel login = _userRepository.Login(loginModel);
            if (login != null && !string.IsNullOrWhiteSpace(login.Token))
                _cache.SetNewUser(login.Username, login.Token);

            return login;
        }

        /// <summary>
        /// LogOut user
        /// </summary>
        /// <param name="username"></param>
        /// <returns></returns>
        [HttpGet]
        public bool LogOut(string username)
        {
            return _userRepository.LogOut(username);
        }
    }
}
