﻿using Microsoft.Extensions.Caching.Memory;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Claims;
using System.Threading;
using System.Threading.Tasks;

namespace IrController.Api.LocalStorage
{
    public class MyCache
    {
        private static string path = $"{AppContext.BaseDirectory.ToString()}cache\\";

        public MemoryCache Cache { get; set; }

        public MyCache()
        {
            Cache = new MemoryCache(new MemoryCacheOptions
            {
                //SizeLimit = 1024
            });
        }

        /// <summary>
        /// Prida nove vytvoreneho uzivatele do cache
        /// </summary>
        /// <param name="username"></param>
        /// <param name="token"></param>
        public void SetNewUser(string username, string token)
        {
            List<UserCache> userCaches = new List<UserCache>();
            bool isExist = Cache.TryGetValue(CacheKeys.UserTokens, out userCaches);

            if (userCaches == null)
                userCaches = new List<UserCache>();

            UserCache cacheUser = userCaches.FirstOrDefault(x => x.Username == username);

            if (cacheUser != null)
                userCaches.Remove(cacheUser);
            userCaches.Add(new UserCache { Token = token, Username = username });


            Cache.Set(CacheKeys.UserTokens, userCaches);
            SaveCache(CacheKeys.UserTokens, DateTime.UtcNow);
        }

        /// <summary>
        /// Overi token uzivatele 
        ///   - false - pokud neni token v cache
        ///   - true - pokud je v cache (a nastavi posledni pouziti - DatetimeLastUsed)
        /// </summary>
        /// <param name="token"></param>
        /// <returns></returns>
        public bool VerifyToken(string token)
        {
            List<UserCache> userCaches = new List<UserCache>();
            bool isExist = Cache.TryGetValue(CacheKeys.UserTokens, out userCaches);

            if (isExist)
            {
                UserCache user = userCaches.FirstOrDefault(x => x.Token == token);

                if (user == null)
                    return false;

                var identity = new ClaimsIdentity(user.Username);
                Thread.CurrentPrincipal = new ClaimsPrincipal(identity);

                user.DatetimeLastUsed = DateTime.UtcNow;
                return true;
            }
            return false;
        }

        private void SaveCache(string name, DateTime tableLastModify, int? stationId = null, string tableName = null)
        {
            string cacheName = stationId == null ? name : $"{stationId}_{name}";
            var loadedCache = new object();

            bool isExist = Cache.TryGetValue(cacheName, out loadedCache);

            if (isExist)
            {
                LocalCacheModel localCacheModel = new LocalCacheModel();

                localCacheModel.Data = loadedCache;
                localCacheModel.CacheName = cacheName;
                localCacheModel.TimeStamp = tableLastModify;

                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);

                using (StreamWriter file = File.CreateText($"{path}{cacheName}.json"))
                {
                    JsonSerializer serializer = new JsonSerializer();
                    serializer.Serialize(file, localCacheModel);
                }
            }
        }


        /// <summary>
        /// Nacte cache ze souboru a porovna, jestli s timestampem v DB jestli je aktualni.
        /// Neni-li aktualni, nacte novou z DB a ulozi do Locakalni
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public bool LoadUserTokensCache()
        {

            string file = $"{path}{CacheKeys.UserTokens}.json";
            if (File.Exists(file))
            {
                using (StreamReader r = new StreamReader(file))
                {
                    string json = r.ReadToEnd();
                    LocalCacheModel item = JsonConvert.DeserializeObject<LocalCacheModel>(json);
                    Cache.Set(item.CacheName, JsonConvert.DeserializeObject<List<UserCache>>(item.Data.ToString()));
                }

            }
            else
            {
                CreateFolderIfNotExist();
            }
            return true;
        }

        private void CreateFolderIfNotExist()
        {
            bool folderExists = Directory.Exists(path);
            if (!folderExists)
                Directory.CreateDirectory(path);
        }
    }
}
