﻿using IrController.Api.Model;
using IrController.Repository;
using Microsoft.AspNetCore.Http;
using Newtonsoft.Json;
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace IrController.Api.Middleware
{
    public class ExceptionMiddleware
    {
        private readonly RequestDelegate _next;
        private readonly ILogRepository _logger;

        public ExceptionMiddleware(RequestDelegate next, ILogRepository logger)
        {
            _next = next;
            _logger = logger;
        }

        public async Task Invoke(HttpContext context)
        {
            try
            {
                if (context == null)
                    return;

                context.Request.EnableBuffering();

                await _next.Invoke(context).ConfigureAwait(false);
            }
            catch (Exception ex)
            {
                using (var reader = new StreamReader(
                context.Request.Body,
                encoding: Encoding.UTF8,
                detectEncodingFromByteOrderMarks: false,
                bufferSize: -1,
                leaveOpen: true))
                {
                    context.Request.Body.Position = 0;
                    var body = await reader.ReadToEndAsync();
                    // Do some processing with body…
                    _logger.AddLog($"|URL:{context.Request.Path}{context.Request.QueryString.ToString()}{Environment.NewLine}" +
                        $"BODY:{body}{Environment.NewLine}" +
                        $"{ex.ToString()}" +
                        $"{Environment.NewLine}{Environment.NewLine}{Environment.NewLine}");
                    // Reset the request body stream position so the next middleware can read it
                    context.Request.Body.Position = 0;
                }
                await HandleExceptionAsync(context, ex).ConfigureAwait(false);
            }
        }

        private static async Task HandleExceptionAsync(HttpContext context, Exception exception)
        {
            if (context == null)
                return;

            InternalErrorException internalErrorException = new InternalErrorException
            {
                Message = exception.Message,
                Request = context.Request.Path,
                Method = context.Request.Method
            };

            var response = context.Response;

            response.ContentType = "application/json";
            response.StatusCode = (int)HttpStatusCode.InternalServerError;
            await response.WriteAsync(JsonConvert.SerializeObject(internalErrorException)).ConfigureAwait(false);
        }
    }
}