﻿using IrController.Api.LocalStorage;
using Microsoft.AspNetCore.Http;
using System.Threading.Tasks;

namespace IrController.Api.Middleware.Security
{
    public class AuthenticationMiddleware
    {
        private readonly RequestDelegate _next;
        private MyCache _cache;

        public AuthenticationMiddleware(RequestDelegate next, MyCache memoryCache)
        {
            _next = next;
            _cache = memoryCache;
        }

        public async Task Invoke(HttpContext context)
        {
            if (context == null)
                return;

            if (context.Request.Path == "/api/User/Login" ||
                context.Request.Path.ToString().Contains("/api/Code")
                )
            {
                await _next.Invoke(context).ConfigureAwait(false);
            }
            else
            {
                string authHeader = context.Request.Headers["Authorization"];
                if (authHeader != null)
                {
                    authHeader = authHeader.Replace("token ", "");
                    if (_cache.VerifyToken(authHeader))
                    {
                        await _next.Invoke(context).ConfigureAwait(false);
                    }
                    else
                    {
                        context.Response.StatusCode = 401; //Unauthorized
                        return;
                    }
                }
                else
                {
                    context.Response.StatusCode = 401; //Unauthorized
                    return;
                }
            }
        }
    }
}
