﻿using IrController.DbModel.Db.Model;
using Microsoft.EntityFrameworkCore;

namespace IrController.DbModel.Db
{
    public class IrControllerContext : DbContext
    {
        public IrControllerContext(DbContextOptions<IrControllerContext> options) : base(options)
        {
        }

        public DbSet<Device> Devices { get; set; }
        public virtual DbSet<User> Users { get; set; }
        public virtual DbSet<Code> Codes { get; set; }
        public virtual DbSet<MissingCode> MissingCodes { get; set; }
        public virtual DbSet<Arduino> Arduinos { get; set; }

        public virtual DbSet<Log> Logs { get; set; }

        public virtual DbSet<DeviceFunction> DeviceFunctions { get; set;}
        public virtual DbSet<DeviceTemplate> DeviceTemplates { get; set; }

        public virtual DbSet<Device_Templates_x_Functions> Device_Templates_x_Functions { get; set; }
        public virtual DbSet<DeviceCategory> DeviceCategories { get; set; }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<DeviceCategory>().HasData(
                new DeviceCategory { Id = 1, CodeString = "switch", Name = "Switch" },
                new DeviceCategory { Id = 2, CodeString = "brightness", Name = "Brightness" },
                new DeviceCategory { Id = 3, CodeString = "color", Name = "Color" }
            );

            modelBuilder.Entity<DeviceFunction>().HasData(
                 new DeviceFunction { Id = 1, CodeString = "powerOn", Name = "On", DeviceCategoryId = 1, Url = "/Code/SetSwitch?value=1" },
                 new DeviceFunction { Id = 2, CodeString = "powerOff", Name = "Off", DeviceCategoryId = 1, Url = "/Code/SetSwitch?value=0" },
                 new DeviceFunction { Id = 3, CodeString = "url", Name = "Set Brightness", DeviceCategoryId = 2, Url = "/Code/SetBrightness?value=%s" },
                 new DeviceFunction { Id = 4, CodeString = "url", Name = "Set Color", DeviceCategoryId = 3, Url = "/Code/SetColor?value=%s" }
             );

            modelBuilder.Entity<DeviceTemplate>().HasData(
                new DeviceTemplate { Id = 1, Name = "Simple Light", Accessory = "HttpPushRgb", Service = "Light" },
                new DeviceTemplate { Id = 2, Name = "Brighness Light", Accessory = "HttpPushRgb", Service = "Light" },
                new DeviceTemplate { Id = 3, Name = "Color Light", Accessory = "HttpPushRgb", Service = "Light" }
            );


            modelBuilder.Entity<Device_Templates_x_Functions>().HasData(
                new Device_Templates_x_Functions { Id = 1, DeviceFunctionId = 1, DeviceTemplateId = 1 },
                new Device_Templates_x_Functions { Id = 2, DeviceFunctionId = 2, DeviceTemplateId = 1 },

                new Device_Templates_x_Functions { Id = 3, DeviceFunctionId = 1, DeviceTemplateId = 2 },
                new Device_Templates_x_Functions { Id = 4, DeviceFunctionId = 2, DeviceTemplateId = 2 },
                new Device_Templates_x_Functions { Id = 5, DeviceFunctionId = 3, DeviceTemplateId = 2 },

                new Device_Templates_x_Functions { Id = 6, DeviceFunctionId = 1, DeviceTemplateId = 3 },
                new Device_Templates_x_Functions { Id = 7, DeviceFunctionId = 2, DeviceTemplateId = 3 },
                new Device_Templates_x_Functions { Id = 8, DeviceFunctionId = 3, DeviceTemplateId = 3 },
                new Device_Templates_x_Functions { Id = 9, DeviceFunctionId = 4, DeviceTemplateId = 3 }
            );

            modelBuilder.Entity<Arduino>().HasData(
                new Arduino { Id = 1, Name = "Arduino Test", Ip = "192.168.0.41", Mac = "AA:AA:AA:AA:AA:AB", Token = "Zu6xoYVAHgla03fz" },
                new Arduino { Id = 2, Name = "Arduino Obyvak", Ip = "192.168.0.40", Mac = "AA:AA:AA:AA:AA:AA", Token = "TlpwLwjOBaaQstkf" }
            );

            modelBuilder.Entity<Device>().HasData(
                new Device { Id = 1, Name = "Hlavni svetlo", ArduinoId = 1, DeviceTemplateId = 1 },
                new Device { Id = 2, Name = "Barevne svetlo", ArduinoId = 2, DeviceTemplateId = 3, MaxBrightnessStep = 4 }
            );

            modelBuilder.Entity<Code>().HasData(
                new Code { Id = 1, Name = "Rozsviceni", IrCommand = "0x3", SAddress = "0xEF00", DeviceId = 1, DeviceFunctionId = 1, IrProtocol = "NEC" },
                new Code { Id = 2, Name = "Zhasnuti", IrCommand = "0x2", SAddress = "0xEF00", DeviceId = 1, DeviceFunctionId = 2, IrProtocol = "NEC" },

                new Code { Id = 3, Name = "Rozsviceni", IrCommand = "0x3", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 1, IrProtocol = "NEC" },
                new Code { Id = 4, Name = "Zhasnuti", IrCommand = "0x2", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 2, IrProtocol = "NEC" },
                new Code { Id = 5, Name = "Stmivani +", IrCommand = "0x0", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 3, IrProtocol = "NEC", BrightnessUp = true },
                new Code { Id = 6, Name = "Stmivani -", IrCommand = "0x1", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 3, IrProtocol = "NEC" },
                new Code { Id = 7, Name = "Cervena", IrCommand = "0x4", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 4, IrProtocol = "NEC", ColorValue = "990000" },
                new Code { Id = 8, Name = "Zelena", IrCommand = "0x5", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 4, IrProtocol = "NEC", ColorValue = "009900" },
                new Code { Id = 9, Name = "Modra", IrCommand = "0x6", SAddress = "0xEF00", DeviceId = 2, DeviceFunctionId = 4, IrProtocol = "NEC", ColorValue = "000099" }
            );

            modelBuilder.Entity<User>().HasData(
                new User { Id = new System.Guid("a41610fc-fdc0-41ee-be7e-ac49df18a0bf"), Token = "test", Username = "Admin", Password = "FaUoOCtBLUjIBzWVEfLSUZ6zSu2iodG2le7+CDjcz8Y=" } //PSW: Admin
            );

        }


    }
}
