﻿using System;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace IrController.DbModel.Db.Model
{
    /// <summary>
    /// Known Command
    /// </summary>
    public class Code
    {
        [Key]
        public int Id { get; set; }

        /// <summary>
        /// Name of IR signal
        /// </summary>
        /// 
        /// <example>
        /// Power on, Power off
        /// </example>
        public string Name { get; set; }

        /// <summary>
        /// Type of IR signal
        /// </summary>
        /// <example>
        /// NEC
        /// </example>
        public string IrProtocol { get; set; }

        /// <summary>
        /// If is Brightness up
        /// </summary>
        public bool BrightnessUp { get; set; }

        /// <summary>
        /// String name of RGB color
        /// </summary>
        public string ColorValue { get; set; }

        public string SAddress { get; set; }

        /// <summary>
        /// IR value in HEX
        /// </summary>
        /// 
        /// <example>
        /// 0x7, 0x2
        /// </example>
        public string IrCommand { get; set; }


        public int DeviceId { get; set; }

        public string Status { get; set; }

        /// <summary>
        /// Device name
        /// </summary>
        /// 
        /// <example>
        /// TV, main lamp,...
        /// </example>
        [ForeignKey("DeviceId")]
        [InverseProperty("Codes")]
        public virtual Device Device { get; set; }


        /// <summary>
        /// Helper for generating json accessories
        /// </summary>
        public int? DeviceFunctionId { get; set; }
        [ForeignKey("DeviceFunctionId")]
        [InverseProperty("Codes")]
        public virtual DeviceFunction DeviceFunction { get; set; }
    }
}
