﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace IrController.DbModel.Db.Model
{
    /// <summary>
    /// Known device
    /// </summary>
    public class Device
    {
        public Device()
        {
            Codes = new List<Code>();
        }
         
        [Key]
        public int Id { get; set; }

        /// <summary>
        /// Name of Device
        /// </summary>
        /// 
        /// <example>
        /// TV, Main lamp....
        /// </example>
        public string Name { get; set; }

        /// <summary>
        /// Maximum steps to full Brightness
        /// </summary>
        /// <example>
        /// If user must click 10 times from lowest Brghtness to full it will be 10
        /// </example>
        public int? MaxBrightnessStep { get; set; }

        

        /// <summary>
        /// Codes for this device
        /// </summary>
        /// 
        /// <example>
        /// Power on, Power off, ...
        /// </example>
        [InverseProperty("Device")]
        public ICollection<Code> Codes { get; set; }

        /// <summary>
        /// Connection for Arduino for sending to specific IP. If is not set it will be send to all Arduinos
        /// </summary>
        public int? ArduinoId { get; set; }
        [ForeignKey("ArduinoId")]
        [InverseProperty("Devices")]
        public virtual Arduino Arduino { get; set; }

        public int DeviceTemplateId { get; set; }
        [ForeignKey("DeviceTemplateId")]
        [InverseProperty("Devices")]
        public virtual DeviceTemplate DeviceTemplate { get; set; }
    }
}
