﻿using IrController.DbModel.Db;
using IrController.DbModel.Db.Model;
using IrController.Model;
using IrController.Repository.Factories;
using System;
using System.Collections.Generic;
using System.Linq;
using IrController.Repository.Helpers;

namespace IrController.Repository
{
    [Attributes.LoadRepository]
    public class ArduinoRepository : IArduinoRepository
    {
        private readonly IIrControllerContextFactory _dbContextFactory;
        public ArduinoRepository(IIrControllerContextFactory dbContextFactory)
        {
            _dbContextFactory = dbContextFactory;
        }

        public bool ManageArduino(ArduinoModel arduino)
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                Arduino dbArduino = new Arduino();
                if (arduino.Id < 1)
                {
                    dbArduino.Token = HashHelper.CalculateHash();
                    _db.Arduinos.Add(dbArduino);
                }
                else
                {
                    dbArduino = _db.Arduinos.First(x => x.Id == arduino.Id);
                    
                }

                dbArduino.Name = arduino.Name;
                dbArduino.Ip = arduino.Ip;
                dbArduino.Mac = arduino.Mac;

                try
                {
                    _db.SaveChanges();
                    return true;
                }
                catch (Exception e)
                {
                    return false;
                }

            }
        }

        public List<ArduinoModel> GetArduinos()
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                return _db.Arduinos.Select(x => new ArduinoModel
                {
                    Id = x.Id,
                    Name = x.Name,
                    Ip = x.Ip,
                    Mac = x.Mac,
                    DevicesCount = x.Devices.Count()
                }).ToList();
            }
        }

        public string RemoveArduino(int id)
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                int countArduinos = 0;
                try
                {
                    countArduinos = _db.Arduinos.First(x => x.Id == id).Devices.Count;
                }
                catch (Exception)
                {
                    return "Arduino not found!";
                }

                if (countArduinos > 1)
                    return "Arduino contain Devices, please remove codes first";

                try
                {
                    _db.Arduinos.Remove(_db.Arduinos.First(x => x.Id == id));
                    _db.SaveChanges();
                }
                catch (Exception e)
                {
                    return e.Message;
                }

                return "";

            }
        }

        public ArduinoModel GetArduino(int id)
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                return _db.Arduinos.Select(x => new ArduinoModel
                {
                    Id = x.Id,
                    Name = x.Name,
                    Ip = x.Ip,
                    Mac = x.Mac,
                    DevicesCount = x.Devices.Count()
                }).First(x => x.Id == id);
            }
        }

        public List<ArduinoSimple> GetSimpleArduinos()
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                return _db.Arduinos.Select(x => new ArduinoSimple
                {
                    Id = x.Id,
                    Name = x.Name
                }).ToList();
            }
        }
    }
}
