﻿using IrController.Model;
using System.Collections.Generic;

namespace IrController.Repository
{
    public interface ICodeRepository
    {
        /// <summary>
        /// Adding new Code to DB
        /// </summary>
        /// <param name="irProtocol"></param>
        /// <param name="sAddress"></param>
        /// <param name="irCommand"></param>
        /// <returns></returns>
        bool GetNewCode(string com, string add, string code);

        /// <summary>
        /// Edit code which was already assign
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        bool ManageCode(CodeModel code);

        /// <summary>
        /// Write command from Arduino to DB
        /// </summary>
        /// <param name="command"></param>
        /// <returns></returns>
        bool WriteCommand(string com, string add, string irp);

        /// <summary>
        /// Return all codes
        /// </summary>
        /// <returns></returns>
        List<CodeModel> GetAllCodes();

        /// <summary>
        /// Return single code for edit
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        CodeModel GetCode(int id);

        /// <summary>
        /// Return all codes for single device
        /// </summary>
        /// <param name="deviceId"></param>
        /// <returns></returns>
        List<CodeModel> GetCoddesByDevice(int deviceId);

        /// <summary>
        /// Return all function assigned to device
        /// </summary>
        /// <param name="deviceId"></param>
        /// <returns></returns>
        List<DeviceFunctionModel> GetDeviceFunctions(int deviceId);

        /// <summary>
        /// Management of missing code
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        CodeModel GetMissingCode(int id);

        /// <summary>
        /// Return all missing codes
        /// </summary>
        /// <returns></returns>
        public List<CodeModel> GetAllMissingCodes();

        /// <summary>
        /// Removing code
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        string RemoveCode(int id);

        /// <summary>
        /// Removing missing code
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        string RemoveMissingCode(int id);

        /// <summary>
        /// Adding Missing code to Code
        /// </summary>
        /// <param name="code"></param>
        /// <returns></returns>
        bool ManageMissingCode(CodeModel code);

        bool SetSwitch(int value, int codeId);

        bool SetColor(string value, int codeId);
        bool SetBrightness(int value, int codeId);

        string GetStatus(int codeId);
    }
}