﻿using IrController.DbModel.Db;
using IrController.DbModel.Db.Model;
using IrController.Model.User;
using IrController.Repository.Factories;
using Microsoft.AspNetCore.Cryptography.KeyDerivation;
using System;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading;

namespace IrController.Repository
{
    [Attributes.LoadRepository]
    public class UserRepository : IUserRepository
    {
        private readonly IIrControllerContextFactory _dbContextFactory;
        public UserRepository(IIrControllerContextFactory dbContextFactory)
        {
            _dbContextFactory = dbContextFactory;
        }

        public LogedUserModel ChangePassword(NewPasswordModel changePassword)
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                string username = Thread.CurrentPrincipal?.Identity.Name;
                string hashed = CalculatePassword(changePassword.Password, username);
                User user = _db.Users.FirstOrDefault(x => x.Username == username && x.Password == hashed);

                if (user == null)
                    return new LogedUserModel();

                user.Token = CalculateToken();
                user.Password = CalculatePassword(changePassword.Password, username);

                _db.SaveChanges();

                return new LogedUserModel
                {
                    Token = user.Token,
                    Username = user.Username
                };
            }
        }


        public LogedUserModel Login(LoginModel loginModel)
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                string hashed = CalculatePassword(loginModel.Password, loginModel.Username);
                User user = _db.Users.FirstOrDefault(x => x.Username == loginModel.Username && x.Password == hashed);

                if (user == null)
                    return new LogedUserModel();

                user.Token = CalculateToken();
                _db.SaveChanges();

                return new LogedUserModel
                {
                    Token = user.Token,
                    Username = user.Username
                };
            }
        }


        public bool LogOut(string username)
        {
            using (IrControllerContext _db = _dbContextFactory.CreateDbContext())
            {
                User user = _db.Users.First(x => x.Username == username);

                if (user != null)
                {
                    user.Token = "";
                    _db.SaveChanges();
                }
            }
            return true;
        }

        private string CalculatePassword(string password, string username)
        {
            byte[] salt = new byte[128 / username.Count()];

            string hashed = Convert.ToBase64String(KeyDerivation.Pbkdf2(
                password: password,
                salt: salt,
                prf: KeyDerivationPrf.HMACSHA512,
                iterationCount: 10000,
                numBytesRequested: 256 / 8));

            return hashed;
        }

        private string CalculateToken()
        {
            // Convert the salted password to a byte array
            byte[] saltedHashBytes = Encoding.UTF8.GetBytes(Guid.NewGuid().ToString());
            // Use the hash algorithm to calculate the hash
            HashAlgorithm algorithm = new SHA256Managed();
            byte[] hash = algorithm.ComputeHash(saltedHashBytes);

            algorithm.Dispose();
            // Return the hash as a base64 encoded string to be compared to the stored password
            return Convert.ToBase64String(hash);
        }
    }
}
